CON
    _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

  ' Button Pins
  Play          = 16
  Back          = 17
  Forward       = 18
  Up            = 19
  Down          = 20
  Stop          = 21              
  Record        = 22
  Library       = 23
  Menu          = 24
  Power         = 25

  GenreFile = 1
  ArtistFile = 2
  AlbumFile = 3
  SongFile = 4

  ' Status variable bit mask                        
  Status_SD_Mask                = %10000000             ' Bit mask for SD permission bit in the status varable.
  Status_Pause_Mask             = %00000001             ' Bit mask for pause bit in the status variable.
  Status_Stop_Mask              = %00000010             ' Bit mask for stop bit in the status variable.
  Status_Audio_Mask             = %00000011             ' Bit mask for both the stop and pause bits in the status variable.
  Status_ClearProgress_Mask     = %00100000             ' Bit mask for progress bar update bit.
  Status_UpdateText_Mask        = %01000000             ' Bit mask for text update bit
  Status_LockInput_Mask         = %00001000             ' Bit mask for Lock Input bit

  CLK20ms                       = 1596068               ' Number of clock ticks in 20 ms

  ' Addresses of data in the EEPROM
  GenreAddr                     = $7000                 ' Starting address of the song database and the Genre file in the EEPROM
  

OBJ
  SDCard     : "fsrwFemto"
  Screen     : "uOLED_96"
  MP3        : "vs1002 mp3"
  Numbers    : "Simple_Numbers"
  Serial     : "FullDuplexSerial"   

VAR
  Byte MP3_Cog                  ' IDs of used cogs
  Byte Button_Cog
  Byte Desplay_Cog

  Long MP3Stack[150]            ' Cog Stacks
  Long ButtonStack[150]
  Long DesplayStack[150]

  Word PlayListLength 
  Word PlayList[512]           ' Database variables
  
  Word NowPlaying               ' Current Song informaiton 
  byte NowPlayingName[12]
  Long SongLength
  Long PlayTime
  
  byte Status                   ' Bits: [SD Access] [Update Text] [Clear Progress] [Lock Input] [Unused] [Stop] [Pause]
  Byte InvertLeftStatus
  Byte BassBoostValue
  Byte FreqLimitValue
  Byte HighPass
  Byte RandomStatus
  Long CurrentBalance
                                                                              
  Byte SongName[256]            ' Used to store the name of the currently playing song
  Byte AlbumName[256]           ' Used to store the album name of the currently playing song
  Byte ArtistName[256]          ' Used to store the artist name of the currently playing song
  Byte NameBuffer[256]   

  byte SelectedMenuLine
  Word SelectedMenuItem
  Word SearchItems[516]
  Long SearchItemsLength

  Long LastGenre, LastArtist, LastAlbum

  Word SelectedGenre, SelectedArtist, SelectedAlbum

  Long ArtistAddr, AlbumAddr, SongAddr
  Byte EEPROMBuf

  long ioControl[2]

  long tp, expr
  long Buffer     

PUB Main | Count
  Serial.Start(31,30,0,9600)
  
'******************
'** Setup EEPROM **
'******************
  SDCard.Start(@ioControl)
  
'************************
'** Setup uOLED Screen **
'************************
  Screen.Init                   ' Initalize the uOLED Object
  Screen.Font_Size(0)           ' Set the font size to 5x7
  Screen.Background(255,255,255)
  Screen.Text_Mode(1)           ' Set the font mode to Opaque

  Screen.Contrast(ScreenContrast)

  Screen.Image(24,8,48,32,16,@Splash)
  Screen.Text_Center(45,0,0,0,String("PropMP3"))
  Screen.Text_Center(45,0,0,0,String("Loading"))
 
'**************************
'** Setup SD Card Object **
'**************************
  if \SDCard.mount(8,9,10,11) <> 0      ' Start SD SPI Cog and mount the first partition
    Screen.Erase
    Screen.Text_Center(20,0,0,0,String("Error"))                                   
    Screen.Text_Center(28,0,0,0,String("Can not read"))
    Screen.Text_Center(36,0,0,0,String("SD card"))
    waitcnt(cnt + (clkfreq * 3))
    PowerDown
  
'****************************
'** Load Library To EEPROM **
'****************************
     
  ArtistAddr := LoadFile(String("Genre.Bin"),GenreAddr)
     
  AlbumAddr := LoadFile(String("Artist.Bin"),ArtistAddr)

  SongAddr := LoadFile(string("Album.Bin"),AlbumAddr)
      
  LoadFile(String("Songs.Bin"),SongAddr)

  Count~
  repeat 480
    Serial.tx(SDCard.readEEPROMByte(SongAddr+Count++))
    
  Screen.Paint(0,0, 95,63, 255,255,255)
  Screen.Text_Center(0,0,0,0,String("Now Playing "))

  Screen.FChar(2,7,0,0,0,":")
  Screen.FChar(13,7,0,0,0,":")

  Screen.Line(0,8,96,8,0,0,0)
  Screen.Image(2,37,11,10,8,@VolImage)
  Screen.Image(83,0,12,8,8,@Batt)
                                                                                                                   
  Screen.Rectangle(5,49,91,54,0,0,0,0)
  Screen.Rectangle(15,40,91,45,0,0,0,0)

'**********************
'** Setup Mp3 Object **
'**********************
  MP3.Start(0,1,2,3,4)          ' Start the communication Cog

  Count := cnt + clkfreq
  repeat until ina[5] == 1      ' Wait until the vs1002 chip is ready.
    if cnt == Count
      Screen.Erase
      Screen.Text_Center(20,0,0,0,String("Error"))                                   
      Screen.Text_Center(28,0,0,0,String("MP3 Decoder"))
      Screen.Text_Center(36,0,0,0,String("Error"))
      waitcnt(cnt + (clkfreq * 3))
      PowerDown
  
  MP3.SetVolume(Volume,CurrentBalance) 'Read volume from the EEPROM and set the chip volume to that.
  MP3.SetMode(MP3#DIFF,InvertLeftStatus)                   'Set Left chanle inverter
  MP3.SetBassBoost(BassBoostValue)
  MP3.SetFreqLimit(FreqLimitValue)
  MP3.SetMode(MP3#ADPCM_HP,HighPass)                             

  Status := %00000010 

  MP3_Cog := CogNew(PlayMp3,@MP3Stack)
  Button_Cog := CogNew(ButtonLoop,@ButtonStack)
  Desplay_Cog := CogNew(Desplay,@DesplayStack)

  OpenMP3File(PlayList[NowPlaying],1)
  
  Timer
    
PUB Timer
  
  repeat
    repeat 1000
      Waitcnt((clkfreq / 1000) + cnt)
      if ina[Library] == 0                                ' If the Power switch is turned off
        PowerDown
      if Status <> 0
        repeat until Status == 0
          if ina[Library] == 0                                ' If the Power switch is turned off
            PowerDown
    PlayTime += 1
  
PUB Desplay | LastVolume
  LastVolume~
  repeat     
    Print_Time(0,7,PlayTime)
    Screen.Paint(6,50,((((PLayTime*90) * 1000) / (SongLength * 1000)) + 6) <# 90,53,0,155,0)

    if MP3.Volume > LastVolume 
      Screen.Paint(16,41,((((Mp3.Volume-70) * 74) / 185) + 16),44,255,128,0)

    if MP3.Volume < LastVolume 
      Screen.Paint(((((Mp3.Volume-70) * 74) / 185) + 16),41,90,44,255,255,255) ' Clear volume bar

    LastVolume := Mp3.Volume
     
    Screen.Image(0,0,8,8,8,@PlayImage + (((Status & Status_Audio_Mask) <# 2) * 64))

    if Status & Status_UpdateText_Mask
      Status ^= Status_LockInput_Mask
      Print_Song_Info(PlayList[NowPlaying])
      Status ^= Status_LockInput_Mask       
      Status ^= Status_UpdateText_Mask

    if Status & Status_ClearProgress_Mask
      Screen.Paint(6,50,90,53,255,255,255)
      PlayTime := 0
      Status ^= Status_ClearProgress_Mask

                        
PUB ButtonLoop | rnd
  
  repeat
    repeat until Status & Status_LockInput_Mask == 0
    if ina[Play] == 0                                   ' If the Play/Pause button is pressed
      if Status & Status_Stop_Mask                      ' If the player is stoped
        Status &= !Status_Audio_Mask                    ' And start playing
      else 
        Status ^= Status_Pause_Mask                     ' Invert the pause bit in Status variable
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1                       ' Wait until the button is released

    if ina[Up] == 0                                     ' if Up button is pressed
      MP3.SetVolume((MP3.Volume + 4) <# 255,CurrentBalance) ' Turn up the Volume
      Volume+=4
      SDCard.WriteEEPROM((SDCard#bootAddr | (@Volume & $7FFFF)),@Volume,1)' Write Volume to EEPROM
      Waitcnt(cnt + (clkfreq / 8))                      ' Give the mp3 cog a chance to send data

    if ina[Down] == 0                                   ' If the Down button is pressed
      MP3.SetVolume(MP3.Volume - 4 #> 70,CurrentBalance) ' Turn down the volume
      Volume-=4
      SDCard.WriteEEPROM((SDCard#bootAddr | (@Volume & $7FFFF)),@Volume,1)' Write Volume to EEPROM 
      Waitcnt(cnt + (clkfreq / 8))

    if ina[Stop] == 0                                   ' If the stop button is pressed
      Status |= Status_Audio_Mask                       ' Set the Stop bit in Status
      NowPlaying~                                       ' Return NowPlaying to zero
      SDCard.pclose
      OpenMP3File(PlayList[NowPlaying],0)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1                       ' Wait until the Stop button is released.                      
      
    if ina[Back] == 0                                   ' If the previous button is pressed
      if RandomStatus <> 0
        rnd := ||?cnt // 12
        repeat while rnd == NowPlaying
          rnd := ||?cnt // 12
        NowPlaying := rnd
        OpenMP3File(NowPlaying,1)
      else    
        if NowPlaying > 0                                 ' If we are not at the beginning of the play list
          OpenMP3File(PlayList[--NowPlaying],1)           ' Load the new song. 
      
      repeat until ina[Back] == 1                       ' Wait until the button is released
      waitcnt(CLK20ms + cnt)
  

    if ina[Forward] == 0                                ' If the next button is pressed
      if RandomStatus <> 0
        rnd := ||?cnt // 12
        repeat while rnd == NowPlaying
          rnd := ||?cnt // 12
        NowPlaying := rnd
        OpenMP3File(NowPlaying,1)
      else  
        if NowPlaying < PlayListLength                  ' If we are not at the end of the play list then
          OpenMP3File(PlayList[++NowPlaying],1)         ' Load the new song
        
      repeat until ina[Forward] == 1                    ' Wait until the button is released
      waitcnt(CLK20ms + cnt)
                                                                                       
    if ina[Menu] == 0                                   ' If the Menu button is pressed
      CogStop(Desplay_Cog)
      MainMenu

    if ina[Record] == 0                                   ' If the Menu button is pressed
      CogStop(Desplay_Cog)
      LibrarySongs(0)                                       

    if ina[Library] == 0                                ' If the Power switch is turned off
      PowerDown       
 
PUB OpenMP3File(Song_ID,UpdateText)
  repeat until (Status & Status_SD_Mask) == 0           ' Wait for permition to use the sd card.
  Status |= Status_SD_Mask                              ' Take control of sd card. 
  waitcnt(cnt + 1000)                                   ' Wait for the mp3 cog to stop

  Get_Song_FileName(Song_ID,@NowPlayingName)            ' Get the file name of the song

  SongLength := Get_Song_Length(Song_ID)                ' Get song length from file

  ByteFill(@SongName,0,255)
  ByteFill(@AlbumName,0,255)
  ByteFill(@ArtistName,0,255)
  
  Get_Song_Name(Song_ID,@SongName)                      ' Get song name and put it in the song name string
  Get_Song_Album(Song_ID,@AlbumName)                    ' Get the album name of the current song and put it in the album string
  Get_Song_Artist(Song_ID,@ArtistName)                  ' Get artist of the current song from the database and put it in the artist string

  Status |= Status_ClearProgress_Mask                   ' Clear the Progress bar
  
  if UpdateText <> 0                                    ' If we want to update the song text
    Status |= Status_UpdateText_Mask                 
  
  SDCard.pOpen(@NowPlayingName,"r")                     ' Open the new mp3 file
  Status &= !Status_SD_Mask                             ' Release control of sd card

PUB PlayMP3 | rnd
  repeat                  
    if Status & %10000011 == 0                          ' If it is not paused then countinue

      if ina[5] == 1                                    ' If the mp3 chip wants more data then
        
        if SDCard.pRead(@SDBuffer,32) =< 0              ' First read 32 bytes into a buffer
          if RandomStatus <> 0
            rnd := ||?cnt // 12
            repeat while rnd == NowPlaying
              rnd := ||?cnt // 12
            NowPlaying := rnd
            OpenMP3File(NowPlaying,1)
          else    
            if NowPlaying < PlayListLength              ' If we are not at the end of the play list then
              OpenMP3File(PlayList[++NowPlaying],1)     ' Load the new song
            else
              Status |= Status_Audio_Mask               ' Set the Stop bit in Status
              NowPlaying~                               ' Return NowPlaying to zero
              SDCard.pclose
              OpenMP3File(PlayList[NowPlaying],0)                  
                                                                     
        MP3.WriteDataBuffer(@SDBuffer)                  ' Write the buffer to the mp3 chip.

PUB PowerDown | Count
  MP3.SetVolume(0,0)
  repeat Count from 0 to 4 
    SDCard.WriteEEPROM((SDCard#bootAddr | (@PlayList + (Count * 128) & $7FFFF)),@PlayList + (Count * 128),128)    'Save Play List
    SDCard.WriteWait(SDCard#bootAddr | (@PlayList + (Count * 128) & $7FFFF))
    
  SDCard.WriteEEPROM((SDCard#bootAddr | (@PlayListLength & $7FFFF)),@PlayListLength,2)
  Status := %00000110                                   ' Stop writing to the screen
  Screen.Shutdown                                       ' Turn off the OLED screen
  MP3.Stop                                              ' Stop MP3 ASM Cog
  CogStop(MP3_Cog)                                      ' Stop Spin MP3 Cog
  SDCard.pClose
  SDCard.unMount
  SDCard.Stop                                           ' Stop the sd card cog  
  CogStop(Desplay_Cog)                                  ' Stop the Desplay
  CogStop(0)                                            ' Stop the main cog
  CogStop(Button_Cog)                                   ' Stop the button cog (This one) 

CON
    
''*************************************
''*************************************
''**********               ************
''**********   Menu Code   ************
''**********               ************
''*************************************
''*************************************

PUB MainMenu
  CogStop(Desplay_Cog)
  Screen.Paint(0,0, 95,63, 255,255,255)
  Screen.Text_Mode(0)
  
  Screen.Text_Center(0,0,0,0,String("Menu"))
  Screen.Line(0,8,96,8,0,0,0)

  SelectedMenuLine~
  
  ByteMove(@MenuBuffer,String("Contrast"),17)
  ByteMove(@MenuBuffer + 17,String("Left Invert"),17)
  ByteMove(@MenuBuffer + 34,String("Balance"),17)
  ByteMove(@MenuBuffer + 51,String("Rec High-Pass"),17)
  ByteMove(@MenuBuffer + 68,String("Randomizer"),17)
  ByteMove(@MenuBuffer + 85,String("Bass Boost"),17)
  ByteMove(@MenuBuffer + 102,String("Freq Limit"),17)

  UpdateMenu(SelectedMenuLine)
  
  repeat
    if ina[Up] == 0
      SelectedMenuLine := SelectedMenuLine - 1 #> 0
      UpdateMenuPosition(SelectedMenuLine,SelectedMenuLine + 1)
      waitcnt(CLK20ms + cnt)                 
      repeat until ina[Up] == 1

    elseif ina[Down] == 0
      SelectedMenuLine := SelectedMenuLine + 1 <# 7
      UpdateMenuPosition(SelectedMenuLine,SelectedMenuLine - 1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Down] == 1

    elseif ina[Play] == 0
      Case SelectedMenuLine
        0 : Contrast
        1 : LeftInvert
        2 : balance
        3 : RecHighPass
        4 : Randomizer
        5 : BassBoost
        6 : FreqLimit
        
    elseif ina[stop] == 0
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      ButtonLoop

PUB Contrast | NewContrast
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Contrast"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.Image(1,29,8,8,8,@ContrastEmpty)
  Screen.Image(87,29,8,8,8,@ContrastFull)

  NewContrast := ScreenContrast
  
  Screen.Rectangle(3,38,91,43,0,0,0,0)
  Screen.Paint(4,39,4 + NewContrast * 6 <# 88,42,255,128,0)
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Forward] == 0) or (ina[Up] == 0)
      NewContrast := NewContrast + 1 <# 15
      Screen.Contrast(NewContrast)
      Screen.Paint(4,39,NewContrast * 6 <# 90,42,255,128,0)
      waitcnt((clkfreq / 8) + cnt)

    if (ina[Back] == 0) or (ina[Down] == 0)
      NewContrast := NewContrast - 1 #> 0
      Screen.Contrast(NewContrast)
      Screen.Paint(4 + (NewContrast * 6 <# 90),39,90,42,255,255,255)
      waitcnt((clkfreq / 8) + cnt)

    if ina[Play] == 0
      ScreenContrast := NewContrast
 
      SDCard.WriteEEPROM((SDCard#bootAddr | (@ScreenContrast & $7FFFF)),@ScreenContrast,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
      
    if ina[stop] == 0
      Screen.Contrast(ScreenContrast)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu
  
PUB LeftInvert | NewVal
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Left Invert"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.Text_Center(46,0,0,0,String("Simulates"))
  Screen.Text_Center(55,0,0,0,String("surround sound."))

  Screen.UText(48,17,0,0,0,String("ON"))
  Screen.UText(48,28,0,0,0,String("OFF"))

  NewVal := InvertLeftStatus
  
  Screen.Image(36,17,8,8,8,@SelectedFalse + (NewVal * 64))
  Screen.Image(36,28,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Up] == 0) or (ina[down] == 0) or (ina[back] == 0) or (ina[forward] == 0)
      NewVal ^= %00000001
      Screen.Image(36,17,8,8,8,@SelectedFalse + (NewVal * 64))
      Screen.Image(36,28,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
      MP3.SetMode(MP3#DIFF,NewVal)
      waitcnt(CLK20ms + cnt)
      repeat until (ina[Up] == 1) and (ina[down] == 1) and (ina[back] == 1) and (ina[forward] == 1)

    if ina[Play] == 0
      InvertLeftStatus := NewVal
      MP3.SetMode(MP3#DIFF,InvertLeftStatus)
      SDCard.WriteEEPROM((SDCard#bootAddr | (@InvertLeftStatus & $7FFFF)),@InvertLeftStatus,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
    
    if ina[stop] == 0
      MP3.SetMode(MP3#DIFF,InvertLeftStatus)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu

PUB Balance | OldBalance
  Screen.Paint(0,0, 95,63, 255,255,255)
  Screen.Text_Mode(1)
  Screen.Text_Center(0,0,0,0,String("Balance"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.Rectangle(7,38,89,43,0,0,0,0)
  Screen.UChar(5,29,0,0,0,"L",1,1)
  Screen.UChar(87,29,0,0,0,"R",1,1)

  if CurrentBalance < 0
    Screen.Paint(48 + CurrentBalance,39,48,42,255,128,0)
  else
    Screen.Paint(48,39,48 + CurrentBalance,42,255,128,0)

  Screen.Paint(20,29,60,37,255,255,255)  
  Screen.Text_Center(29,0,0,0,Numbers.dec(CurrentBalance))

  OldBalance := CurrentBalance
  repeat until ina[Play] == 1
                      
  repeat
    if ina[Back] == 0
      CurrentBalance := (CurrentBalance - 1) #> -20
      MP3.SetVolume(MP3.Volume,CurrentBalance)
      Screen.Paint((48 + (CurrentBalance * 2) <# 48) #> 8,39,49,42,255,128,0)
      Screen.Paint(49 + (CurrentBalance * 2) #> 48,39,88,42,255,255,255)
      Screen.Paint(20,29,60,37,255,255,255)
      Screen.Text_Center(29,0,0,0,Numbers.dec(CurrentBalance))
      waitcnt(CLK20ms + cnt)
      repeat until ina[Back] == 1

    if ina[Forward] == 0
      CurrentBalance := (CurrentBalance + 1) <# 20
      MP3.SetVolume(MP3.Volume,CurrentBalance)
      Screen.Paint(48,39,49 + (CurrentBalance * 2 #> 0) <# 88,42,255,128,0)
      Screen.Paint(8,39,47 + CurrentBalance * 2 <# 47,42,255,255,255)
      Screen.Paint(20,29,60,37,255,255,255)      
      Screen.Text_Center(29,0,0,0,Numbers.dec(CurrentBalance))
      waitcnt(CLK20ms + cnt)
      repeat until ina[Forward] == 1

    if ina[Play] == 0
      SDCard.WriteEEPROM((SDCard#bootAddr | (@CurrentBalance & $7FFFF)),@CurrentBalance,4)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu
    
    if ina[stop] == 0
      CurrentBalance := OldBalance
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu
  
PUB RecHighPass | NewVal
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Rec High-Pass"))
  Screen.Line(0,8,96,8,0,0,0)
  Screen.UText(48,22,0,0,0,String("ON"))
  Screen.UText(48,33,0,0,0,String("OFF"))

  NewVal := HighPass <# 1
  
  Screen.Image(36,22,8,8,8,@SelectedFalse + (NewVal * 64))
  Screen.Image(36,33,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
  
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Up] == 0) or (ina[down] == 0) or (ina[back] == 0) or (ina[forward] == 0)
      NewVal ^= %00000001
      Screen.Image(36,22,8,8,8,@SelectedFalse + (NewVal * 64))
      Screen.Image(36,33,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
      MP3.SetMode(MP3#ADPCM_HP,NewVal)
      waitcnt(CLK20ms + cnt)
      repeat until (ina[Up] == 1) and (ina[down] == 1) and (ina[back] == 1) and (ina[forward] == 1)

    if ina[Play] == 0
      HighPass := NewVal
      MP3.SetMode(MP3#ADPCM_HP,HighPass)
      SDCard.WriteEEPROM((SDCard#bootAddr | (@HighPass & $7FFFF)),@HighPass,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
    
    if ina[stop] == 0
      MP3.SetMode(MP3#ADPCM_HP,HighPass)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu

PUB BassBoost | NewBass
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Bass Boost"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.UChar(2,29,0,0,0,"0",1,1)
  Screen.UChar(81,29,0,0,0,"1",1,1)
  Screen.UChar(86,29,0,0,0,"5",1,1)
  
  Screen.Rectangle(2,38,91,43,0,0,0,0)

  NewBass := BassBoostValue
  
  Screen.Paint(3,39,3 + NewBass * 6,42,255,128,0)
  
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Forward] == 0) or (ina[Up] == 0)
      NewBass := NewBass + 1 <# 15
      MP3.SetBassBoost(NewBass)
      Screen.Paint(3,39,NewBass * 6 <# 90,42,255,128,0)
      waitcnt((clkfreq / 8) + cnt)

    if (ina[Back] == 0) or (ina[Down] == 0)
      NewBass := NewBass - 1 #> 0
      MP3.SetBassBoost(NewBass)
      Screen.Paint(4 + (NewBass * 6 <# 90),39,90,42,255,255,255)
      waitcnt((clkfreq / 8) + cnt)

    if ina[Play] == 0
      BassBoostvalue := NewBass
      SDCard.WriteEEPROM((SDCard#bootAddr | (@BassBoostValue & $7FFFF)),@BassBoostValue,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
      
    if ina[stop] == 0
      MP3.SetBassBoost(BassBoostValue)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu

PUB FreqLimit | NewFreq
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Frequency Limit"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.UText(5,29,0,0,0,String("20"))
  Screen.UText(71,29,0,0,0,String("130"))
  
  Screen.Rectangle(8,38,85,43,0,0,0,0)

  NewFreq := FreqLimitValue #> 2
  
  Screen.Paint(9,39,9 + (NewFreq - 2) * 6,42,255,128,0)
  
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Forward] == 0) or (ina[Up] == 0)
      NewFreq := NewFreq + 1 <# 15
      MP3.SetFreqLimit(NewFreq)
      Screen.Paint(9,39,9 + (NewFreq - 2) * 6 <# 84,42,255,128,0)
      waitcnt((clkfreq / 8) + cnt)

    if (ina[Back] == 0) or (ina[Down] == 0)
      NewFreq := NewFreq - 1 #> 2
      MP3.SetFreqLimit(NewFreq)
      Screen.Paint(9 + ((NewFreq - 2) * 6 <# 84),39,84,42,255,255,255)
      waitcnt((clkfreq / 8) + cnt)

    if ina[Play] == 0
      FreqLimitValue := NewFreq
      SDCard.WriteEEPROM((SDCard#bootAddr | (@FreqLimitValue & $7FFFF)),@FreqLimitValue,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
      
    if ina[stop] == 0
      MP3.SetFreqLimit(FreqLimitValue)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu                   
 
PUB Randomizer | NewVal
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.Text_Center(0,0,0,0,String("Left Invert"))
  Screen.Line(0,8,96,8,0,0,0)

  Screen.Text_Center(46,0,0,0,String("Play songs in"))
  Screen.Text_Center(55,0,0,0,String("random order."))

  Screen.UText(48,17,0,0,0,String("ON"))
  Screen.UText(48,28,0,0,0,String("OFF"))

  NewVal := RandomStatus
  
  Screen.Image(36,17,8,8,8,@SelectedFalse + (NewVal * 64))
  Screen.Image(36,28,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
  repeat until ina[Play] == 1
  
  repeat
    if (ina[Up] == 0) or (ina[down] == 0) or (ina[back] == 0) or (ina[forward] == 0)
      NewVal ^= %00000001
      Screen.Image(36,17,8,8,8,@SelectedFalse + (NewVal * 64))
      Screen.Image(36,28,8,8,8,@SelectedFalse + ((NewVal ^ %00000001) * 64))
      waitcnt(CLK20ms + cnt)
      repeat until (ina[Up] == 1) and (ina[down] == 1) and (ina[back] == 1) and (ina[forward] == 1)

    if ina[Play] == 0
      RandomStatus := NewVal
      SDCard.WriteEEPROM((SDCard#bootAddr | (@RandomStatus & $7FFFF)),@RandomStatus,1)
      waitcnt(CLK20ms + cnt)
      repeat until ina[Play] == 1
      MainMenu
    
    if ina[stop] == 0
      waitcnt(CLK20ms + cnt)
      repeat until ina[Stop] == 1
      MainMenu
  
PUB UpdateMenu(SelectedLine) | count
  Screen.Paint(0,9, 95,63, 255,255,255)
  Screen.Paint(0,9 + (SelectedLine * 8),95,16 + (SelectedLine * 8),0,155,0)
  repeat Count from 0 to 6
    Screen.UText(0,9 + (Count * 8),0,0,0,@MenuBuffer + (Count * 17))

PUB UpdateMenuPosition(SelectedLine,OldPoss)
  Screen.Paint(0,9 + (OldPoss * 8),95,16 + (OldPoss * 8),255,255,255)
  Screen.UText(0,9 + (OldPoss * 8),0,0,0,@MenuBuffer + (OldPoss * 17))

  Screen.Paint(0,9 + (SelectedLine * 8),95,16 + (SelectedLine * 8),0,155,0)
  Screen.UText(0,9 + (SelectedLine * 8),0,0,0,@MenuBuffer + (SelectedLine * 17))
  
CON

''********************************
''********************************
''******                    ******
''******    Library Code    ******    
''******                    ******
''********************************
''********************************

PUB LibrarySongs(PreviousMenu) | Count
  CogStop(Desplay_Cog)
  Screen.Paint(0,0, 95,63, 255,255,255)

  Screen.Text_Mode(0)

  Screen.Image(29,0,29,7,8,@SongsImg)

  Screen.Line(0,7,96,7,0,0,0)

  if PreviousMenu == 0
    SelectedGenre := SelectedArtist := SelectedAlbum := LastGenre := LastArtist := LastAlbum := 0

  SelectedMenuLine := 0

  ' Clear Menu Buffer
  Bytefill(@MenuBuffer,0,119)

  wordfill(@SearchItems,0,516)
  SearchItemsLength := -1
  
  if SelectedGenre == 0
    SearchItems[++SearchItemsLength] := 513

  if SelectedArtist == 0
    SearchItems[++SearchItemsLength] := 514

  if SelectedAlbum == 0
    SearchItems[++SearchItemsLength] := 515

  Get_Song_List(SelectedGenre,SelectedArtist,SelectedAlbum)

  SelectedMenuItem := 0
    
  repeat Count From 0 to 6
    if SearchItems[Count] == 513
      ByteMove(@MenuBuffer + (Count * 17),String("Genre"),16)
        
    elseif SearchItems[Count] == 514
      ByteMove(@MenuBuffer + (Count * 17),String("Artist"),16)
  
    elseif SearchItems[Count] == 515
      ByteMove(@MenuBuffer + (Count * 17),String("Album"),16)

    else
      Get_Song_Name(SearchItems[Count],@NameBuffer)
      ByteMove(@MenuBuffer + (Count * 17),@Namebuffer,16)

  Screen.Paint(0,8, 95,63, 255,255,255)
  Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8),0,155,0)
  repeat Count from 0 to 6
    Screen.Text(0,Count+1,0,0,0,0,@MenuBuffer + (Count * 17))
      
  
  repeat
    if (ina[Up] == 0) and (SelectedMenuItem > 0) ' If the Up button is pressed and we are not at top of the menu
      LibraryViewUp(SongFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[UP] == 1)

    elseif (ina[Down] == 0) and (SelectedMenuItem < SearchItemsLength) ' If the Down button is pressed and we are not at the end of the menu
      LibraryViewDown(SongFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[Down] == 1)

    elseif ina[Play] == 0 or ina[Forward] == 0
      repeat until ina[Play] == 1 and ina[Forward] == 1
      case SearchItems[SelectedMenuItem]
        513   : LibraryGenre(PreviousMenu)
        514   : LibraryArtist(PreviousMenu)
        515   : LibraryAlbum(PreviousMenu)
        other : SongTasks(PreviousMenu)

    elseif ina[Back] == 0
      repeat until ina[Back] == 1
      case PreviousMenu
        1 : LibraryGenre(-1)
        2 : LibraryArtist(-1)
        3 : LibraryAlbum(-1)
      
    elseif ina[Stop] == 0                                   ' If the stop button is pressed
      repeat until ina[Stop] == 1
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop

PUB LibraryGenre(PreviousMenu) | Count
  if PreviousMenu <> -1
    LastGenre := PreviousMenu

  Screen.Paint(0,0, 95,63, 255,255,255)

  Screen.Image(29,0,38,6,8,@GenreImg)
  
  Screen.Line(0,7,96,7,0,0,0)

  SelectedMenuLine := 0
  SelectedMenuItem := 0

  ' Clear Menu Buffer
  Bytefill(@MenuBuffer,0,119)
  wordfill(@SearchItems,0,516)
  SearchItemsLength := -1
  
  Get_Genre_List(SelectedArtist,SelectedAlbum)
    
  repeat Count From 0 to 6
    Get_Genre_Name(SearchItems[Count],@NameBuffer)
    ByteMove(@MenuBuffer + (Count * 17),@Namebuffer,16)

  Screen.Paint(0,8, 95,63, 255,255,255)
  Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8),0,155,0)
  repeat Count from 0 to 6
    Screen.Text(0,Count+1,0,0,0,0,@MenuBuffer + (Count * 17))
      
  
  repeat
    if (ina[Up] == 0) and (SelectedMenuItem > 0) ' If the Up button is pressed and we are not at top of the menu
      LibraryViewUp(GenreFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[UP] == 1)

    elseif (ina[Down] == 0) and (SelectedMenuItem < SearchItemsLength) ' If the Down button is pressed and we are not at the end of the menu
      LibraryViewDown(GenreFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[Down] == 1)

    elseif ina[Play] == 0 or ina[Forward] == 0
      SelectedGenre := SearchItems[SelectedMenuItem]
      repeat until ina[Play] == 1 and ina[Forward] == 1
      LibrarySongs(1)

    elseif ina[Back] == 0
      repeat until ina[Back] == 1
      SelectedGenre := 0
      LibrarySongs(LastGenre)
      
    elseif ina[Stop] == 0                                   ' If the stop button is pressed
      repeat until ina[Stop] == 1
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop

PUB LibraryArtist(PreviousMenu) | Count
  if PreviousMenu <> -1
    LastArtist := PreviousMenu

  Screen.Paint(0,0, 95,63, 255,255,255)
    
  Screen.Paint(0,0, 95,63, 255,255,255)

  Screen.Image(29,0,38,6,8,@ArtistImg)
  
  Screen.Line(0,7,96,7,0,0,0)

  SelectedMenuLine := 0
  SelectedMenuItem := 0

  ' Clear Menu Buffer
  Bytefill(@MenuBuffer,0,119)
  wordfill(@SearchItems,0,516)
  SearchItemsLength := -1
  
  Get_Artist_List(SelectedGenre,SelectedAlbum)
    
  repeat Count From 0 to 6
    Get_Artist_Name(SearchItems[Count],@NameBuffer)
    ByteMove(@MenuBuffer + (Count * 17),@Namebuffer,16)

  Screen.Paint(0,8, 95,63, 255,255,255)
  Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8),0,155,0)
  repeat Count from 0 to 6
    Screen.Text(0,Count+1,0,0,0,0,@MenuBuffer + (Count * 17))
      
  
  repeat
    if (ina[Up] == 0) and (SelectedMenuItem > 0) ' If the Up button is pressed and we are not at top of the menu
      LibraryViewUp(ArtistFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[UP] == 1)

    elseif (ina[Down] == 0) and (SelectedMenuItem < SearchItemsLength) ' If the Down button is pressed and we are not at the end of the menu
      LibraryViewDown(ArtistFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[Down] == 1)

    elseif ina[Play] == 0 or ina[Forward] == 0
      SelectedArtist := SearchItems[SelectedMenuItem]
      repeat until ina[Play] == 1 and ina[Forward] == 1
      LibrarySongs(2)

    elseif ina[Back] == 0
      repeat until ina[Back] == 1
      SelectedArtist := 0
      LibrarySongs(LastArtist)
      
    elseif ina[Stop] == 0                                   ' If the stop button is pressed
      repeat until ina[Stop] == 1
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop

PUB LibraryAlbum(PreviousMenu) | Count
  if PreviousMenu <> -1
    LastAlbum := PreviousMenu

  Screen.Paint(0,0, 95,63, 255,255,255)

  Screen.Paint(0,0, 95,63, 255,255,255)

  Screen.Image(29,0,32,6,8,@AlbumImg)
  
  Screen.Line(0,7,96,7,0,0,0)

  ' Clear Menu Buffer
  Bytefill(@MenuBuffer,0,119)
  wordfill(@SearchItems,0,516)
  SearchItemsLength := -1
  
  Get_Album_List(SelectedGenre,SelectedArtist)

  SelectedMenuItem := 0
  SelectedMenuLine := 0
    
  repeat Count From 0 to 6
    Get_Album_Name(SearchItems[Count],@NameBuffer)
    ByteMove(@MenuBuffer + (Count * 17),@Namebuffer,16)

  Screen.Paint(0,8, 95,63, 255,255,255)
  Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8),0,155,0)
  repeat Count from 0 to 6
    Screen.Text(0,Count+1,0,0,0,0,@MenuBuffer + (Count * 17))
      
  
  repeat
    if (ina[Up] == 0) and (SelectedMenuItem > 0) ' If the Up button is pressed and we are not at top of the menu
      LibraryViewUp(AlbumFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[UP] == 1)

    elseif (ina[Down] == 0) and (SelectedMenuItem < SearchItemsLength) ' If the Down button is pressed and we are not at the end of the menu
      LibraryViewDown(AlbumFile)
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[Down] == 1)

    elseif ina[Play] == 0 or ina[Forward] == 0
      SelectedAlbum := SearchItems[SelectedMenuItem]
      repeat until ina[Play] == 1 and ina[Forward] == 1
      LibrarySongs(3)

    elseif ina[Back] == 0
      repeat until ina[Back] == 1
      SelectedAlbum := 0
      LibrarySongs(LastAlbum)
      
    elseif ina[Stop] == 0                                   ' If the stop button is pressed
      repeat until ina[Stop] == 1
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop      

PUB SongTasks(PreviousMenu) | Count, StartingAddr
  Screen.Paint(11,15,84,48,255,255,255)
  Screen.Rectangle(10,14,85,49,0,0,0,0)
  Screen.Rectangle(9,13,86,50,200,200,200,0)
  
  StartingAddr~
  if SelectedGenre == 0
    StartingAddr++
  if SelectedArtist == 0
    StartingAddr++
  if SelectedAlbum == 0
    StartingAddr++

  SelectedMenuLine~
  
  ByteMove(@MenuBuffer,String("Play Now"),17)
  ByteMove(@MenuBuffer + 17,String("Play All"),17)
  ByteMove(@MenuBuffer + 34,String("Add to Queue"),17)
  ByteMove(@MenuBuffer + 51,String("Queue All"),17)

  Screen.Paint(11,15,84,23,0,155,0)
  repeat Count from 0 to 3
    Screen.Text(2,Count+2,0,0,0,0,@MenuBuffer + (Count * 17))

  repeat
    if (ina[Up] == 0) and (SelectedMenuLine > 0) ' If the Up button is pressed and we are not at top of the menu
      SelectedMenuLine--
      Screen.Paint(11,15,84,48,255,255,255)
      Case SelectedMenuLine
        0 : Screen.Paint(11,15,84,23,0,155,0)
        1 : Screen.Paint(11,24,84,31,0,155,0)
        2 : Screen.Paint(11,32,84,39,0,155,0)
        3 : Screen.Paint(11,40,84,48,0,155,0)
      repeat Count from 0 to 3
        Screen.Text(2,Count+2,0,0,0,0,@MenuBuffer + (Count * 17))
      
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[UP] == 1)  

    elseif (ina[Down] == 0) and (SelectedMenuLine < 3) ' If the Down button is pressed and we are not at the end of the menu
      SelectedMenuLine++
      Screen.Paint(11,15,84,48,255,255,255)
      Case SelectedMenuLine
        0 : Screen.Paint(11,15,84,23,0,155,0)
        1 : Screen.Paint(11,24,84,31,0,155,0)
        2 : Screen.Paint(11,32,84,39,0,155,0)
        3 : Screen.Paint(11,40,84,48,0,155,0)
      repeat Count from 0 to 3
        Screen.Text(2,Count+2,0,0,0,0,@MenuBuffer + (Count * 17))
      Count := cnt + (Clkfreq / 5)
      repeat until (cnt > Count) or (ina[Down] == 1)

    elseif ina[Play] == 0 or ina[Forward] == 0
      repeat until ina[Play] == 1 and ina[Forward] == 1

      Case SelectedMenuLine
        0 : PlayListLength := 0 
            PlayList[0] := SearchItems[SelectedMenuItem]
            NowPlaying := 0
            OpenMP3File(PlayList[NowPlaying],0)
            Status &= !Status_Audio_Mask
        
        1 : PlayListLength := (SearchItemsLength - StartingAddr) 
            Wordmove(@PlayList,@SearchItems[StartingAddr],(SearchItemsLength - StartingAddr))
            NowPlaying := 0
            OpenMP3File(PlayList[NowPlaying],0)
            Status &= !Status_Audio_Mask

        2 : PlayList[++PlayListLength] := SearchItems[SelectedMenuItem]
        

        3 : WordMove(@PlayList[PlayListLength + 1],@SearchItems[StartingAddr],(SearchItemsLength - StartingAddr))
            PlayListLength += (SearchItemsLength - StartingAddr)

      repeat Count from 0 to PlayListLength
        Serial.dec(PlayList[Count])
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop

    elseif ina[Back] == 0
      repeat until ina[Back] == 1
      LibrarySongs(LastAlbum)
      
    elseif ina[Stop] == 0                                   ' If the stop button is pressed
      repeat until ina[Stop] == 1
      Status |= Status_UpdateText_Mask
      Setup_Play_Screen
      waitcnt(CLK20ms + cnt)
      ButtonLoop                      

PUB LibraryViewDown(DatabaseFile)
  if SelectedMenuLine < 6
    Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8),255,255,255)
    Screen.Text(0,SelectedMenuLine+1,0,0,0,0,@MenuBuffer + (SelectedMenuLine * 17))

    SelectedMenuLine++
    
    Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8) <# 63,0,155,0)
    Screen.Text(0,SelectedMenuLine+1,0,0,0,0,@MenuBuffer + (SelectedMenuLine * 17))

    SelectedMenuItem++

  else
    ' Update menu buffer

    ByteMove(@NameBuffer,@MenuBuffer + 17,102)
    ByteMove(@MenuBuffer,@NameBuffer,102)
    Bytefill(@MenuBuffer+102,0,17)

    Screen.Block_Copy(0,16,95,55,0,8)
    

    SelectedMenuItem++

    Case DatabaseFile
      GenreFile  : Get_Genre_Name(SearchItems[SelectedMenuItem],@NameBuffer)
      ArtistFile : Get_Artist_Name(SearchItems[SelectedMenuItem],@NameBuffer)
      AlbumFile  : Get_Album_Name(SearchItems[SelectedMenuItem],@NameBuffer)
      SongFile   : Get_Song_Name(SearchItems[SelectedMenuItem],@NameBuffer)
      
    ByteMove(@MenuBuffer + 102,@Namebuffer,16)

    Screen.Paint(0,48,95,56,255,255,255)
    Screen.Paint(0,56,95,63,0,155,0)
    Screen.Text(0,6,0,0,0,0,@MenuBuffer + 85)
    Screen.Text(0,7,0,0,0,0,@MenuBuffer + 102)

PUB LibraryViewUp(DatabaseFile)
  if SelectedMenuLine > 0
    Screen.Paint(0,8 + (SelectedMenuLine *8),95,15 + (SelectedMenuLine * 8),255,255,255)
    Screen.Text(0,SelectedMenuLine+1,0,0,0,0,@MenuBuffer + (SelectedMenuLine * 17))

    SelectedMenuLine--
    
    Screen.Paint(0,8 + (SelectedMenuLine * 8),95,15 + (SelectedMenuLine * 8) <# 63,0,155,0)
    Screen.Text(0,SelectedMenuLine+1,0,0,0,0,@MenuBuffer + (SelectedMenuLine * 17))

    SelectedMenuItem--

  else
    ' Update menu buffer

    ByteMove(@NameBuffer,@MenuBuffer,102)
    ByteMove(@MenuBuffer + 17,@NameBuffer,102)
    Bytefill(@MenuBuffer,0,17)

    Screen.Block_Copy(0,16,95,56,0,24)
    

    SelectedMenuItem--

    if SearchItems[SelectedMenuItem] == 513
      ByteMove(@MenuBuffer,String("Genre"),16)
        
    elseif SearchItems[SelectedMenuItem] == 514
      ByteMove(@MenuBuffer,String("Artist"),16)
  
    elseif SearchItems[SelectedMenuItem] == 515
      ByteMove(@MenuBuffer,String("Album"),16)

    else
      Case DatabaseFile
        GenreFile  : Get_Genre_Name(SearchItems[SelectedMenuItem],@NameBuffer)
        ArtistFile : Get_Artist_Name(SearchItems[SelectedMenuItem],@NameBuffer)
        AlbumFile  : Get_Album_Name(SearchItems[SelectedMenuItem],@NameBuffer)
        SongFile   : Get_Song_Name(SearchItems[SelectedMenuItem],@NameBuffer)
      
      ByteMove(@MenuBuffer,@Namebuffer,16)

    Screen.Paint(0,16,95,23,255,255,255)
    Screen.Text(0,2,0,0,0,0,@MenuBuffer + 17)
    Screen.Paint(0,8,95,15,0,155,0)
    Screen.Text(0,1,0,0,0,0,@MenuBuffer)

CON
    
''************************************
''************************************
''******                        ******          
''******   Song Database Code   ******
''******                        ******
''************************************
''************************************
   
PRI LoadFile(NameAddr,Address) | Count,NumOfBytes,Bytes
  '' Loads a file from the SD Card into the EEPROM
  '' at Address. Return the address of the next byte
  '' after the end of the file
  Count~
  
  SDCard.pOpen(NameAddr,"r")
        
  repeat                                               
    NumOfBytes := SDCard.pRead(@SongName,(128 - ((Address+Count) // 128)))

    SDCard.WriteEEPROM(SDCard#bootAddr | ((Address+Count) & $7FFFF),@SongName,NumOfBytes)
    SDCard.writeWait(Address+Count)
    if NumOfBytes < (128 - ((Address+Count) // 128))
      Count+=NumOfbytes
      quit
      
    Count+=NumOfbytes                     
  SDCard.pClose
  Return Address + Count
     
   
PRI Get_Song_FileName(SongID,RetBuf) | CurrSongID
  SDCard.popen(String("Name.bin"),"r")

  repeat SdCard.pGetL
    CurrSongID := SDCard.pGetL
    if CurrSongID == SongID
      ByteFill(RetBuf,0,12)
      SDCard.pRead(RetBuf,SDCard.pGetL)
      return
    else
      SDCard.seek(SDCard.pGetL)

  SDCard.pclose  

PRI Get_Song_Length(SongID) | CurrSongID, Length
  SDCard.popen(String("Songs.bin"),"r")

  repeat SdCard.pGetL
    CurrSongID := SDCard.pGetL
    SDCard.pGetL
    SDCard.pGetL
    SDCard.pGetL 
    SDCard.pGetL
    Length := SDCard.pGetL
    SDCard.seek(SDCard.pGetL)
    if CurrSongID == SongID
      Return Length

  SDCard.pclose

PRI Get_Song_List(Genre,Artist,Album) | CurrSongID, Compare, FilePoss
  FilePoss~
  
  FilePoss := SongAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4)
    Compare := 0
    CurrSongID := SDCard.readEEPROMLong(FilePoss+=4)

    FilePoss+=4

    if (SDCard.readEEPROMLong(FilePoss+=4) == Genre) or (Genre == 0)
      Compare++
    
    if (SDCard.readEEPROMLong(FilePoss+=4) == Artist) or (Artist == 0)
      Compare++

    if (SDCard.readEEPROMLong(FilePoss+=4) == Album) or (Album == 0)
      Compare++  
      
    FilePoss += 4
        
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))

    if Compare == 3
      SearchItems[++SearchItemsLength] := CurrSongID

PRI Get_Genre_List(Artist,Album) | CurrGenreID, Compare, FilePoss
  FilePoss~
  
  FilePoss := GenreAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4)
    Compare := 0
    CurrGenreID := SDCard.readEEPROMLong(FilePoss+=4)
    
    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Artist) or (Artist == 0)
        Compare++

    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Album) or (Album == 0)
        Compare++
        
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))

    if Compare => 2
      SearchItems[++SearchItemsLength] := CurrGenreID

PRI Get_Artist_List(Genre,Album) | CurrArtistID, Compare, FilePoss
  FilePoss~
  
  FilePoss := ArtistAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4)
    Compare := 0
    CurrArtistID := SDCard.readEEPROMLong(FilePoss+=4)
    
    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Genre) or (Genre == 0)
        Compare++

    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Album) or (Album == 0)
        Compare++
        
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))

    if Compare => 2
      SearchItems[++SearchItemsLength] := CurrArtistID 

PRI Get_Album_List(Genre,Artist) | CurrAlbumID, Compare, FilePoss
  FilePoss~
  
  FilePoss := AlbumAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4)
    Compare := 0
    CurrAlbumID := SDCard.readEEPROMLong(FilePoss+=4)
    
    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Genre) or (Genre == 0)
        Compare++

    repeat SDCard.readEEPROMLong(FilePoss+=4)
      if (SDCard.readEEPROMLong(FilePoss+=4) == Artist) or (Artist == 0)
        Compare++
        
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))

    if Compare => 2
      SearchItems[++SearchItemsLength] := CurrAlbumID


PRI Get_Song_Name(SongID,RetBuf) | CurrSongID, FilePoss
  FilePoss~
  
  FilePoss := SongAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrSongID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += 20 
    if CurrSongID == SongID
      ByteFill(RetBuf,0,256)
      SDCard.ReadEEPROM(SDCard#bootAddr | ((FilePoss + 8) & $7FFFF),RetBuf,SDCard.readEEPROMLong(FilePoss+4))
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
      quit
    else
      ByteFill(RetBuf,0,256)
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4)) 
     
PRI Get_Genre_Name(GenreID,RetBuf) | CurrGenreID, FilePoss
  FilePoss~
  
  FilePoss := GenreAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrGenreID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    if CurrGenreID == GenreID
      ByteFill(RetBuf,0,256)
      SDCard.ReadEEPROM(SDCard#bootAddr | ((FilePoss + 8) & $7FFFF),RetBuf,SDCard.readEEPROMLong(FilePoss+4))
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
      quit
    else
      ByteFill(RetBuf,0,256)
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))


PRI Get_Song_Album(SongID,RetBuf) | CurrSongID, Count, AlbumID, FilePoss
  FilePoss~
  
  FilePoss := SongAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrSongID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += 12
    AlbumID := SDCard.ReadEEPROMLong(FilePoss+=4)
    FilePoss+=4
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
    if CurrSongID == SongID
      Get_Album_Name(AlbumID,RetBuf)
      quit

PRI Get_Song_Artist(SongID,RetBuf) | CurrSongID, Count, ArtistID, FilePoss
  FilePoss~
  
  FilePoss := SongAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrSongID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += 12
    ArtistID := SDCard.ReadEEPROMLong(FilePoss+=4)
    FilePoss+=4
    FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
    if CurrSongID == SongID
      Get_Artist_Name(ArtistID,RetBuf)
      quit

PRI Get_Album_Name(AlbumID,RetBuf) | CurrAlbumID, FilePoss
  FilePoss~
  
  FilePoss := AlbumAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrAlbumID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    if CurrAlbumID == AlbumID
      ByteFill(RetBuf,0,256)
      SDCard.ReadEEPROM(SDCard#bootAddr | ((FilePoss + 8) & $7FFFF),RetBuf,SDCard.readEEPROMLong(FilePoss+4))
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
      quit
    else
      ByteFill(RetBuf,0,256)
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4)) 

PRI Get_Artist_Name(ArtistID,RetBuf) | CurrArtistID, FilePoss
  FilePoss~
  
  FilePoss := ArtistAddr - 4
  repeat SDCard.readEEPROMLong(FilePoss+=4) 
    CurrArtistID := SDCard.readEEPROMLong(FilePoss+=4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    FilePoss += (SDCard.readEEPROMLong(FilePoss+=4) * 4)
    if CurrArtistID == ArtistID
      ByteFill(RetBuf,0,256)
      SDCard.ReadEEPROM(SDCard#bootAddr | ((FilePoss + 8) & $7FFFF),RetBuf,SDCard.readEEPROMLong(FilePoss+4))
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))
      quit
    else
      ByteFill(RetBuf,0,256)
      FilePoss += (SDCard.readEEPROMLong(FilePoss += 4))    

CON

''*****************************
''*****************************
''******                 ******
''******   Screen Code   ******
''******                 ******
''*****************************
''*****************************
      

PRI Print_Time(x,y,time)
'' Print time in Min:Sec fromat to a x and y position on the screen.
'' time is in seconds

  Screen.Text(x,y,0,0,0,0,Numbers.Decz((Time / 60),2))
  Screen.Text(x+3,y,0,0,0,0,Numbers.Decz((Time // 60),2))
  return

PRI Print_Song_Info(Song_ID) 

  ' Print song length
  Print_Time(11,7,SongLength)                           ' Print the song length
  
  ' Print the song name
  Screen.Paint(0,11,96,35,255,255,255)
'  Screen.Paint(0,11,96,19,255,255,255)                  ' Clear the old name
  Screen.Text_Center(11,0,0,0,@SongName)                ' Then print the name centered on the screen
  
  ' Print the album name   
'  Screen.Paint(0,20,96,28,255,255,255)                  ' Clear the old album
  Screen.Text_Center(20,0,0,0,@AlbumName)               ' Print the album name to the screen
  
  ' Print the artist name  
'  Screen.Paint(0,29,96,34,255,255,255)                  ' Clear the old artist 
  Screen.Text_Center(29,0,0,0,@ArtistName)              ' Print the new name to the center of the center of the screen

PRI Setup_Play_Screen
  '' Setup all the static parts of the Now Playing screen
  Screen.Text_Mode(1)
  Screen.Paint(0,0, 95,63, 255,255,255)
  
  Screen.FChar(2,7,0,0,0,":")
  Screen.FChar(13,7,0,0,0,":")

  Screen.Line(0,8,96,8,0,0,0)
  Screen.Image(2,37,11,10,8,@VolImage)
  Screen.Image(83,0,12,8,8,@Batt)
                                                                                                                   
  Screen.Rectangle(5,49,91,54,0,0,0,0)
  Screen.Rectangle(15,40,91,45,0,0,0,0)

  Desplay_Cog := CogNew(Desplay,@DesplayStack)
   

DAT
        Volume          byte 180
        ScreenContrast  byte 8


        SDBuffer        byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

        MenuBuffer      byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

        SongsImg        byte $FF, $00, $00, $00, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF,{
                            }$00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $00,{
                            }$00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $FF,{
                            }$FF, $00, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $00, $00, $FF,{
                            }$FF, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $FF, $FF, $00,{        
                            }$00, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $00, $00, $00, $00, $FF,{
                            }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF


        
        GenreImg        byte $FF, $00, $00, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF,{
                            }$00, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $00, $FF, $00, $00, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $00,{
                            }$00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $00, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $FF,{
                            }$00, $FF, $FF, $00, $00, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $FF,{
                            }$00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $00,{
                            }$FF, $00, $00, $00, $FF, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $00, $00, $00, $00, $FF


        ArtistImg       byte $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF,{
                            }$FF, $00, $FF, $00, $FF, $FF, $00, $FF, $00, $00, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $00, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $00, $00, $00, $00,{
                            }$00, $FF, $FF, $FF, $00, $FF, $00, $00, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF,{
                            }$00, $00, $00, $00, $00, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $00, $00, $FF,{ 
                            }$00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00,{
                            }$00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $00, $00, $FF, $FF, $00, $00, $00, $FF, $00, $00, $00, $00, $FF, $FF, $FF, $FF, $00, $00, $FF, $FF, $00, $00, $00, $00, $FF

        AlbumImg       byte $FF, $FF, $00, $FF, $FF, $FF, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF,{  
                            }$FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $00, $FF, $00, $00, $FF, $FF, $00, $00, $00, $00,{  
                            }$00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $00, $00, $FF, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $00, $FF, $00, $FF, $00, $FF, $FF, $FF, $FF,{
                            }$00, $00, $00, $00, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $00, $00, $00, $FF,{
                            }$00, $FF, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $FF, $00, $FF, $00, $FF, $FF, $00, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $FF, $00,{
                            }$00, $FF, $FF, $FF, $00, $FF, $00, $00, $00, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $00, $FF, $00, $FF, $00, $FF, $FF, $FF, $00, $FF, $00, $00, $00, $00, $FF

        VolImage        byte  $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $00, $FF, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $00, $FF, $FF, $00, {
                             }$FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $FF, $00, {
                             }$00, $00, $00, $00, $00, $00, $FF, $FF, $00, $FF, $00, {
                             }$00, $00, $00, $00, $00, $00, $FF, $FF, $00, $FF, $00, {
                             }$FF, $FF, $00, $00, $00, $FF, $FF, $FF, $00, $FF, $00, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $00, $FF, $FF, $00, {
                             }$FF, $FF, $FF, $FF, $00, $FF, $FF, $FF, $FF, $00, $FF, {
                             }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF

        PlayImage       byte  $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $00, $00, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF

        PauseImage      byte  $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF

        StopImage       byte  $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF    

        Batt            byte  $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, {
                             }$FF, $00, $00, $00, $FF, $00, $00, $FF, $00, $00, $FF, $00, {
                             }$00, $00, $00, $FF, $00, $00, $FF, $00, $00, $FF, $00, $00, {
                             }$00, $00, $FF, $00, $00, $FF, $00, $00, $FF, $00, $00, $00, {
                             }$FF, $00, $00, $00, $FF, $00, $00, $FF, $00, $00, $00, $00, {
                             }$FF, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, {
                             }$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF

        ContrastFull    byte  $FF, $FF, $FF, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $FF, $00, $FF, {
                             }$00, $00, $00, $00, $00, $FF, $FF, $00, {
                             }$00, $00, $00, $00, $00, $FF, $FF, $00, {
                             }$FF, $00, $00, $00, $00, $FF, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $FF

        ContrastEmpty   byte  $FF, $FF, $FF, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $FF, $00, $FF, {
                             }$00, $00, $00, $FF, $FF, $FF, $FF, $00, {
                             }$00, $00, $00, $FF, $FF, $FF, $FF, $00, {
                             }$FF, $00, $00, $FF, $FF, $FF, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $FF

       SelectedFalse    byte  $FF, $FF, $FF, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $00, $FF, $FF, $FF, $FF, $00, $FF, {
                             }$00, $FF, $FF, $FF, $FF, $FF, $FF, $00, {
                             }$00, $FF, $FF, $FF, $FF, $FF, $FF, $00, {
                             }$FF, $00, $FF, $FF, $FF, $FF, $00, $FF, {
                             }$FF, $00, $00, $FF, $FF, $00, $00, $FF, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $FF

       SelectedTrue     byte  $FF, $FF, $FF, $00, $00, $FF, $FF, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$00, $00, $00, $00, $00, $00, $00, $00, {
                             }$00, $00, $00, $00, $00, $00, $00, $00, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $00, $00, $00, $00, $00, $00, $FF, {
                             }$FF, $FF, $FF, $00, $00, $FF, $FF, $FF

                                                                                         
                                                
        Splash          word  $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFF, $FFFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00103D, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $00103D, $00103D, $00103D, $FFFFFF, $FFFFFF, $FFFFFF, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $0007C1, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00103D, $00103D, $00103D, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00103D, $00103D, $00103D, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00103D, $00103D, $00103D, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007FC, $0007FC, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $0007FC, $0007FC, $0007FC, $0007FC, $0007FC, $0007FC, $0007FC, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $0007FF, $0007FF, $0007FF, $0007FF, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00095C, $00095C, $00095C, $00095C, $00095C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $00FA3C, $00FA3C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00095C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $0007CF, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $00FA3C, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, {
}                             $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF, $FFFFFF
   